<%# Non-link tag that stands for skipped pages...
  - available local variables
    current_page:  a page object for the currently displayed page
    total_pages:   total number of pages
    per_page:      number of items to fetch per page
    remote:        data-remote
-%>
<span class="page gap"><%= t('views.pagination.truncate').html_safe %></span>
